///<reference path=".d.ts"/>
"use strict";
var path = require("path");
var semver = require("semver");
var AndroidToolsInfo = (function () {
    function AndroidToolsInfo($childProcess, $errors, $fs, $logger, $options) {
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$fs = $fs;
        this.$logger = $logger;
        this.$options = $options;
        this.installedTargetsCache = null;
        this.androidHome = process.env["ANDROID_HOME"];
    }
    AndroidToolsInfo.prototype.getToolsInfo = function () {
        var _this = this;
        return (function () {
            if (!_this.toolsInfo) {
                var infoData = Object.create(null);
                infoData.androidHomeEnvVar = _this.androidHome;
                infoData.compileSdkVersion = _this.getCompileSdk().wait();
                infoData.buildToolsVersion = _this.getBuildToolsVersion().wait();
                infoData.targetSdkVersion = _this.getTargetSdk().wait();
                infoData.supportRepositoryVersion = _this.getAndroidSupportRepositoryVersion().wait();
                _this.toolsInfo = infoData;
            }
            return _this.toolsInfo;
        }).future()();
    };
    AndroidToolsInfo.prototype.validateInfo = function (options) {
        var _this = this;
        return (function () {
            var detectedErrors = false;
            _this.showWarningsAsErrors = options && options.showWarningsAsErrors;
            var toolsInfoData = _this.getToolsInfo().wait();
            if (!toolsInfoData.androidHomeEnvVar || !_this.$fs.exists(toolsInfoData.androidHomeEnvVar).wait()) {
                _this.printMessage("The ANDROID_HOME environment variable is not set or it points to a non-existent directory. You will not be able to perform any build-related operations for Android.", "To be able to perform Android build-related operations, set the ANDROID_HOME variable to point to the root of your Android SDK installation directory.");
            }
            if (!toolsInfoData.compileSdkVersion) {
                _this.printMessage("Cannot find a compatible Android SDK for compilation. To be able to build for Android, install Android SDK " + AndroidToolsInfo.MIN_REQUIRED_COMPILE_TARGET + " or later.", "Run `$ android` to manage your Android SDK versions.");
                detectedErrors = true;
            }
            if (!toolsInfoData.buildToolsVersion) {
                var buildToolsRange = _this.getBuildToolsRange();
                var versionRangeMatches = buildToolsRange.match(/^.*?([\d\.]+)\s+.*?([\d\.]+)$/);
                var message = "You can install any version in the following range: '" + buildToolsRange + "'.";
                if (versionRangeMatches && versionRangeMatches[1] && versionRangeMatches[2] && versionRangeMatches[1] === versionRangeMatches[2]) {
                    message = "You have to install version " + versionRangeMatches[1] + ".";
                }
                _this.printMessage("You need to have the Android SDK Build-tools installed on your system. " + message, 'Run "android" from your command-line to install required Android Build Tools.');
                detectedErrors = true;
            }
            if (!toolsInfoData.supportRepositoryVersion) {
                _this.printMessage("You need to have the latest Android Support Repository installed on your system.", 'Run `$ android`  to manage the Android Support Repository.');
                detectedErrors = true;
            }
            if (options && options.validateTargetSdk) {
                var targetSdk = toolsInfoData.targetSdkVersion;
                var newTarget = AndroidToolsInfo.ANDROID_TARGET_PREFIX + "-" + targetSdk;
                if (!_.contains(AndroidToolsInfo.SUPPORTED_TARGETS, newTarget)) {
                    var supportedVersions = AndroidToolsInfo.SUPPORTED_TARGETS.sort();
                    var minSupportedVersion = _this.parseAndroidSdkString(_.first(supportedVersions));
                    if (targetSdk && (targetSdk < minSupportedVersion)) {
                        _this.printMessage("The selected Android target SDK " + newTarget + " is not supported. You must target " + minSupportedVersion + " or later.");
                        detectedErrors = true;
                    }
                    else if (!targetSdk || targetSdk > _this.getMaxSupportedVersion()) {
                        _this.$logger.warn("Support for the selected Android target SDK " + newTarget + " is not verified. Your Android app might not work as expected.");
                    }
                }
            }
            return detectedErrors;
        }).future()();
    };
    AndroidToolsInfo.prototype.printMessage = function (msg, additionalMsg) {
        if (this.showWarningsAsErrors) {
            this.$errors.failWithoutHelp(msg);
        }
        else {
            this.$logger.warn(msg);
        }
        if (additionalMsg) {
            this.$logger.info(additionalMsg);
        }
    };
    AndroidToolsInfo.prototype.getCompileSdk = function () {
        var _this = this;
        return (function () {
            if (!_this.selectedCompileSdk) {
                var userSpecifiedCompileSdk = _this.$options.compileSdk;
                if (userSpecifiedCompileSdk) {
                    var installedTargets = _this.getInstalledTargets().wait();
                    var androidCompileSdk = AndroidToolsInfo.ANDROID_TARGET_PREFIX + "-" + userSpecifiedCompileSdk;
                    if (!_.contains(installedTargets, androidCompileSdk)) {
                        _this.$errors.failWithoutHelp("You have specified '" + userSpecifiedCompileSdk + "' for compile sdk, but it is not installed on your system.");
                    }
                    _this.selectedCompileSdk = userSpecifiedCompileSdk;
                }
                else {
                    var latestValidAndroidTarget = _this.getLatestValidAndroidTarget().wait();
                    if (latestValidAndroidTarget) {
                        var integerVersion = _this.parseAndroidSdkString(latestValidAndroidTarget);
                        if (integerVersion && integerVersion >= AndroidToolsInfo.MIN_REQUIRED_COMPILE_TARGET) {
                            _this.selectedCompileSdk = integerVersion;
                        }
                    }
                }
            }
            return _this.selectedCompileSdk;
        }).future()();
    };
    AndroidToolsInfo.prototype.getTargetSdk = function () {
        var _this = this;
        return (function () {
            var targetSdk = _this.$options.sdk ? parseInt(_this.$options.sdk) : _this.getCompileSdk().wait();
            _this.$logger.trace("Selected targetSdk is: " + targetSdk);
            return targetSdk;
        }).future()();
    };
    AndroidToolsInfo.prototype.getMatchingDir = function (pathToDir, versionRange) {
        var _this = this;
        return (function () {
            var selectedVersion;
            if (_this.$fs.exists(pathToDir).wait()) {
                var subDirs = _this.$fs.readDirectory(pathToDir).wait()
                    .filter(function (buildTools) { return !!buildTools.match(AndroidToolsInfo.VERSION_REGEX); });
                _this.$logger.trace("Versions found in " + pathToDir + " are " + subDirs.join(", "));
                selectedVersion = semver.maxSatisfying(subDirs, versionRange);
            }
            return selectedVersion;
        }).future()();
    };
    AndroidToolsInfo.prototype.getBuildToolsRange = function () {
        return AndroidToolsInfo.REQUIRED_BUILD_TOOLS_RANGE_PREFIX + " <=" + this.getMaxSupportedVersion();
    };
    AndroidToolsInfo.prototype.getBuildToolsVersion = function () {
        var _this = this;
        return (function () {
            var buildToolsVersion;
            if (_this.androidHome) {
                var pathToBuildTools = path.join(_this.androidHome, "build-tools");
                var buildToolsRange = _this.getBuildToolsRange();
                buildToolsVersion = _this.getMatchingDir(pathToBuildTools, buildToolsRange).wait();
            }
            return buildToolsVersion;
        }).future()();
    };
    AndroidToolsInfo.prototype.getAppCompatRange = function () {
        var _this = this;
        return (function () {
            var compileSdkVersion = _this.getCompileSdk().wait();
            var requiredAppCompatRange;
            if (compileSdkVersion) {
                requiredAppCompatRange = ">=" + compileSdkVersion + " <" + (compileSdkVersion + 1);
            }
            return requiredAppCompatRange;
        }).future()();
    };
    AndroidToolsInfo.prototype.getAndroidSupportRepositoryVersion = function () {
        var _this = this;
        return (function () {
            var selectedAppCompatVersion;
            var requiredAppCompatRange = _this.getAppCompatRange().wait();
            if (_this.androidHome && requiredAppCompatRange) {
                var pathToAppCompat = path.join(_this.androidHome, "extras", "android", "m2repository", "com", "android", "support", "appcompat-v7");
                selectedAppCompatVersion = _this.getMatchingDir(pathToAppCompat, requiredAppCompatRange).wait();
            }
            _this.$logger.trace("Selected AppCompat version is: " + selectedAppCompatVersion);
            return selectedAppCompatVersion;
        }).future()();
    };
    AndroidToolsInfo.prototype.getLatestValidAndroidTarget = function () {
        var _this = this;
        return (function () {
            var installedTargets = _this.getInstalledTargets().wait();
            return _.findLast(AndroidToolsInfo.SUPPORTED_TARGETS.sort(), function (supportedTarget) { return _.contains(installedTargets, supportedTarget); });
        }).future()();
    };
    AndroidToolsInfo.prototype.parseAndroidSdkString = function (androidSdkString) {
        return parseInt(androidSdkString.replace(AndroidToolsInfo.ANDROID_TARGET_PREFIX + "-", ""));
    };
    AndroidToolsInfo.prototype.getInstalledTargets = function () {
        var _this = this;
        return (function () {
            if (!_this.installedTargetsCache) {
                _this.installedTargetsCache = [];
                var output = _this.$childProcess.exec('android list targets').wait();
                output.replace(/id: \d+ or "(.+)"/g, function (m, p1) { return (_this.installedTargetsCache.push(p1), m); });
            }
            return _this.installedTargetsCache;
        }).future()();
    };
    AndroidToolsInfo.prototype.getMaxSupportedVersion = function () {
        return this.parseAndroidSdkString(_.last(AndroidToolsInfo.SUPPORTED_TARGETS.sort()));
    };
    AndroidToolsInfo.ANDROID_TARGET_PREFIX = "android";
    AndroidToolsInfo.SUPPORTED_TARGETS = ["android-17", "android-18", "android-19", "android-21", "android-22", "android-23"];
    AndroidToolsInfo.MIN_REQUIRED_COMPILE_TARGET = 22;
    AndroidToolsInfo.REQUIRED_BUILD_TOOLS_RANGE_PREFIX = ">=22";
    AndroidToolsInfo.VERSION_REGEX = /^(\d+\.){2}\d+$/;
    return AndroidToolsInfo;
})();
exports.AndroidToolsInfo = AndroidToolsInfo;
$injector.register("androidToolsInfo", AndroidToolsInfo);
