///<reference path="../../.d.ts"/>
"use strict";
var Future = require("fibers/future");
var osenv = require("osenv");
var path = require("path");
var shell = require("shelljs");
var IosEmulatorServices = (function () {
    function IosEmulatorServices($logger, $emulatorSettingsService, $errors, $childProcess, $devicePlatformsConstants, $hostInfo, $options, $fs, $bplistParser) {
        this.$logger = $logger;
        this.$emulatorSettingsService = $emulatorSettingsService;
        this.$errors = $errors;
        this.$childProcess = $childProcess;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$hostInfo = $hostInfo;
        this.$options = $options;
        this.$fs = $fs;
        this.$bplistParser = $bplistParser;
    }
    IosEmulatorServices.prototype.checkDependencies = function () {
        return Future.fromResult();
    };
    IosEmulatorServices.prototype.checkAvailability = function (dependsOnProject) {
        var _this = this;
        if (dependsOnProject === void 0) { dependsOnProject = true; }
        return (function () {
            if (!_this.$hostInfo.isDarwin) {
                _this.$errors.fail("iOS Simulator is available only on Mac OS X.");
            }
            var platform = _this.$devicePlatformsConstants.iOS;
            if (dependsOnProject && !_this.$emulatorSettingsService.canStart(platform).wait()) {
                _this.$errors.fail("The current project does not target iOS and cannot be run in the iOS Simulator.");
            }
        }).future()();
    };
    IosEmulatorServices.prototype.startEmulator = function (app, emulatorOptions) {
        var _this = this;
        return (function () {
            _this.killLaunchdSim().wait();
            _this.startEmulatorCore(app, emulatorOptions);
        }).future()();
    };
    IosEmulatorServices.prototype.postDarwinNotification = function (notification) {
        var iosSimPath = require.resolve("ios-sim-portable");
        var nodeCommandName = process.argv[0];
        var opts = ["notify-post", notification];
        if (this.$options.device) {
            opts.push("--device", this.$options.device);
        }
        return this.$childProcess.exec(nodeCommandName + " " + iosSimPath + " " + opts.join(' '));
    };
    IosEmulatorServices.prototype.sync = function (appIdentifier, projectFilesPath, notRunningSimulatorAction) {
        var syncAction = function (applicationPath) { return shell.cp("-Rf", projectFilesPath, applicationPath); };
        return this.syncCore(appIdentifier, notRunningSimulatorAction, syncAction);
    };
    IosEmulatorServices.prototype.syncFiles = function (appIdentifier, projectFilesPath, projectFiles, notRunningSimulatorAction, relativeToProjectBasePathAction) {
        var _this = this;
        var syncAction = function (applicationPath) { return _.each(projectFiles, function (projectFile) {
            var destinationPath = path.join(applicationPath, relativeToProjectBasePathAction(projectFile));
            _this.$logger.trace("Transfering " + projectFile + " to " + destinationPath);
            shell.cp("-Rf", projectFile, destinationPath);
        }); };
        return this.syncCore(appIdentifier, notRunningSimulatorAction, syncAction);
    };
    IosEmulatorServices.prototype.isSimulatorRunning = function () {
        var _this = this;
        return (function () {
            try {
                var output = _this.$childProcess.exec("ps cax | grep launchd_sim").wait();
                return output.indexOf('launchd_sim') !== -1;
            }
            catch (e) {
                return false;
            }
        }).future()();
    };
    IosEmulatorServices.prototype.killLaunchdSim = function () {
        this.$logger.info("Cleaning up before starting the iOS Simulator");
        return this.$childProcess.spawnFromEvent("killall", ["launchd_sim"], "close", undefined, { throwError: false });
    };
    IosEmulatorServices.prototype.startEmulatorCore = function (app, emulatorOptions) {
        this.$logger.info("Starting iOS Simulator");
        var iosSimPath = require.resolve("ios-sim-portable");
        var nodeCommandName = process.argv[0];
        if (this.$options.availableDevices) {
            this.$childProcess.spawnFromEvent(nodeCommandName, [iosSimPath, "device-types"], "close", { stdio: "inherit" }).wait();
            return;
        }
        var opts = [
            iosSimPath,
            "launch", app
        ];
        if (this.$options.timeout) {
            opts = opts.concat("--timeout", this.$options.timeout);
        }
        if (!this.$options.justlaunch) {
            opts.push("--logging");
        }
        else {
            if (emulatorOptions) {
                if (emulatorOptions.stderrFilePath) {
                    opts = opts.concat("--stderr", emulatorOptions.stderrFilePath);
                }
                if (emulatorOptions.stdoutFilePath) {
                    opts = opts.concat("--stdout", emulatorOptions.stdoutFilePath);
                }
            }
            opts.push("--exit");
        }
        if (this.$options.device) {
            opts = opts.concat("--device", this.$options.device);
        }
        else if (emulatorOptions && emulatorOptions.deviceType) {
            opts = opts.concat("--device", emulatorOptions.deviceType);
        }
        if (emulatorOptions && emulatorOptions.args) {
            opts.push("--args=" + emulatorOptions.args);
        }
        this.$childProcess.spawn(nodeCommandName, opts, { stdio: "inherit" });
    };
    IosEmulatorServices.prototype.getRunningSimulatorId = function (appIdentifier) {
        var _this = this;
        return (function () {
            if (_this.$options.device) {
                _this._cachedSimulatorId = _this.$options.device;
            }
            if (!_this._cachedSimulatorId) {
                var output = _this.$childProcess.exec("xcrun simctl list").wait();
                var lines = output.split("\n");
                var regex = /[\s\S]+?\(([0-9A-F\-]+?)\)\s+?\(Booted\)/;
                _.each(lines, function (line) {
                    var match = regex.exec(line);
                    if (match) {
                        _this._cachedSimulatorId = match[1];
                        return false;
                    }
                });
                if (!_this._cachedSimulatorId) {
                    regex = /[\s\S]+?\(([0-9A-F\-]+?)\)\s+?\(Shutdown\)/;
                    _.each(lines, function (line) {
                        var match = regex.exec(line);
                        if (match) {
                            _this._cachedSimulatorId = match[1];
                            return false;
                        }
                    });
                }
            }
            return _this._cachedSimulatorId;
        }).future()();
    };
    IosEmulatorServices.prototype.getApplicationPath = function (appIdentifier, runningSimulatorId) {
        var _this = this;
        return (function () {
            var rootApplicationsPath = path.join(osenv.home(), "/Library/Developer/CoreSimulator/Devices/" + runningSimulatorId + "/data/Containers/Bundle/Application");
            if (!_this.$fs.exists(rootApplicationsPath).wait()) {
                rootApplicationsPath = path.join(osenv.home(), "/Library/Developer/CoreSimulator/Devices/" + runningSimulatorId + "/data/Applications");
            }
            var applicationGuids = _this.$fs.readDirectory(rootApplicationsPath).wait();
            var result = null;
            _.each(applicationGuids, function (applicationGuid) {
                var fullApplicationPath = path.join(rootApplicationsPath, applicationGuid);
                var applicationDirContents = _this.$fs.readDirectory(fullApplicationPath).wait();
                var applicationName = _.find(applicationDirContents, function (fileName) { return path.extname(fileName) === ".app"; });
                var plistFilePath = path.join(fullApplicationPath, applicationName, "Info.plist");
                var applicationData = _this.$bplistParser.parseFile(plistFilePath).wait();
                if (applicationData[0].CFBundleIdentifier === appIdentifier) {
                    result = path.join(fullApplicationPath, applicationName);
                    return false;
                }
            });
            return result;
        }).future()();
    };
    IosEmulatorServices.prototype.syncCore = function (appIdentifier, notRunningSimulatorAction, syncAction) {
        var _this = this;
        return (function () {
            if (!_this.isSimulatorRunning().wait()) {
                notRunningSimulatorAction().wait();
            }
            var runningSimulatorId = _this.getRunningSimulatorId(appIdentifier).wait();
            var applicationPath = _this.getApplicationPath(appIdentifier, runningSimulatorId).wait();
            var applicationName = path.basename(applicationPath);
            syncAction(applicationPath);
            try {
                _this.$childProcess.exec("killall " + applicationName.split(".")[0]).wait();
            }
            catch (e) {
                _this.$logger.trace("Unable to kill simulator: " + e);
            }
            setTimeout(function () {
                _this.$childProcess.exec("xcrun simctl launch " + runningSimulatorId + " " + appIdentifier);
            }, 500);
        }).future()();
    };
    return IosEmulatorServices;
})();
$injector.register("iOSEmulatorServices", IosEmulatorServices);
