///<reference path="../.d.ts"/>
"use strict";
var minimatch = require("minimatch");
var path = require("path");
var util = require("util");
var gaze = require("gaze");
var SyncBatch = (function () {
    function SyncBatch($logger, $dispatcher, done) {
        this.$logger = $logger;
        this.$dispatcher = $dispatcher;
        this.done = done;
        this.timer = null;
        this.syncQueue = [];
    }
    SyncBatch.prototype.addFile = function (filePath) {
        var _this = this;
        if (this.timer) {
            clearTimeout(this.timer);
        }
        this.syncQueue.push(filePath);
        this.timer = setTimeout(function () {
            var filesToSync = _this.syncQueue;
            if (filesToSync.length > 0) {
                _this.syncQueue = [];
                _this.$logger.trace("Syncing %s", filesToSync.join(", "));
                _this.$dispatcher.dispatch(function () {
                    return (function () { return _this.done(filesToSync); }).future()();
                });
            }
            _this.timer = null;
        }, 500);
    };
    Object.defineProperty(SyncBatch.prototype, "syncPending", {
        get: function () {
            return this.syncQueue.length > 0;
        },
        enumerable: true,
        configurable: true
    });
    return SyncBatch;
})();
var UsbLiveSyncServiceBase = (function () {
    function UsbLiveSyncServiceBase($devicesServices, $mobileHelper, $localToDevicePathDataFactory, $logger, $options, $deviceAppDataFactory, $fs, $dispatcher, $injector, $childProcess, $iOSEmulatorServices, $hostInfo) {
        this.$devicesServices = $devicesServices;
        this.$mobileHelper = $mobileHelper;
        this.$localToDevicePathDataFactory = $localToDevicePathDataFactory;
        this.$logger = $logger;
        this.$options = $options;
        this.$deviceAppDataFactory = $deviceAppDataFactory;
        this.$fs = $fs;
        this.$dispatcher = $dispatcher;
        this.$injector = $injector;
        this.$childProcess = $childProcess;
        this.$iOSEmulatorServices = $iOSEmulatorServices;
        this.$hostInfo = $hostInfo;
        this._initialized = false;
        this.batch = null;
    }
    UsbLiveSyncServiceBase.prototype.initialize = function (platform) {
        var _this = this;
        return (function () {
            if (!(_this.$options.emulator && platform && platform.toLowerCase() === "ios")) {
                _this.$devicesServices.initialize({ platform: platform, deviceId: _this.$options.device }).wait();
                _this._initialized = true;
                return _this.$devicesServices.platform;
            }
        }).future()();
    };
    UsbLiveSyncServiceBase.prototype.sync = function (platform, appIdentifier, projectFilesPath, excludedProjectDirsAndFiles, watchGlob, platformSpecificLiveSyncServices, notInstalledAppOnDeviceAction, notRunningiOSSimulatorAction, localProjectRootPath, beforeLiveSyncAction, beforeBatchLiveSyncAction, iOSSimulatorRelativeToProjectBasePathAction) {
        var _this = this;
        return (function () {
            var platformLowerCase = platform.toLowerCase();
            var synciOSSimulator = _this.$hostInfo.isDarwin && platformLowerCase === "ios" && (_this.$options.emulator || _this.$iOSEmulatorServices.isSimulatorRunning().wait());
            if (synciOSSimulator) {
                _this.$iOSEmulatorServices.sync(appIdentifier, projectFilesPath, notRunningiOSSimulatorAction).wait();
            }
            if (!_this._initialized && (!_this.$options.emulator || platform.toLowerCase() === "android")) {
                _this.initialize(platform).wait();
            }
            if (!_this.$options.emulator || platform.toLowerCase() === "android") {
                var projectFiles = _this.$fs.enumerateFilesInDirectorySync(projectFilesPath, function (filePath, stat) { return !_this.isFileExcluded(path.relative(projectFilesPath, filePath), excludedProjectDirsAndFiles, projectFilesPath); }, { enumerateDirectories: true });
                _this.syncCore(platform, projectFiles, appIdentifier, localProjectRootPath || projectFilesPath, platformSpecificLiveSyncServices, notInstalledAppOnDeviceAction, beforeLiveSyncAction).wait();
            }
            if (_this.$options.watch) {
                var that = _this;
                gaze("**/*", { cwd: watchGlob }, function (err, watcher) {
                    this.on('all', function (event, filePath) {
                        if (event === "added" || event === "changed") {
                            if (!_.contains(excludedProjectDirsAndFiles, filePath)) {
                                if (synciOSSimulator) {
                                    that.batchSimulatorLiveSync(appIdentifier, projectFilesPath, filePath, notRunningiOSSimulatorAction, iOSSimulatorRelativeToProjectBasePathAction);
                                }
                                if (!that.$options.emulator || platform.toLowerCase() === "android") {
                                    that.batchLiveSync(platform, filePath, appIdentifier, localProjectRootPath || projectFilesPath, platformSpecificLiveSyncServices, notInstalledAppOnDeviceAction, beforeLiveSyncAction, beforeBatchLiveSyncAction);
                                }
                            }
                        }
                    });
                });
                _this.$dispatcher.run();
            }
        }).future()();
    };
    UsbLiveSyncServiceBase.prototype.syncCore = function (platform, projectFiles, appIdentifier, projectFilesPath, platformSpecificLiveSyncServices, notInstalledAppOnDeviceAction, beforeLiveSyncAction) {
        var _this = this;
        return (function () {
            platform = platform ? _this.$mobileHelper.normalizePlatformName(platform) : _this.$devicesServices.platform;
            var deviceAppData = _this.$deviceAppDataFactory.create(appIdentifier, platform);
            var localToDevicePaths = _(projectFiles)
                .map(function (projectFile) { return _this.getProjectFileInfo(projectFile); })
                .filter(function (projectFileInfo) { return projectFileInfo.shouldIncludeFile; })
                .map(function (projectFileInfo) { return _this.$localToDevicePathDataFactory.create(projectFileInfo.fileName, projectFilesPath, projectFileInfo.onDeviceName, deviceAppData.deviceProjectRootPath); })
                .value();
            var action = function (device) {
                return (function () {
                    if (deviceAppData.isLiveSyncSupported(device).wait()) {
                        if (beforeLiveSyncAction) {
                            beforeLiveSyncAction(device, deviceAppData).wait();
                        }
                        var applications = device.applicationManager.getInstalledApplications().wait();
                        if (!_.contains(applications, deviceAppData.appIdentifier)) {
                            _this.$logger.warn("The application with id \"" + deviceAppData.appIdentifier + "\" is not installed on the device yet.");
                            notInstalledAppOnDeviceAction(device).wait();
                            return;
                        }
                        _this.$logger.info("Transferring project files...");
                        device.fileSystem.transferFiles(deviceAppData.appIdentifier, localToDevicePaths).wait();
                        _this.$logger.info("Successfully transferred all project files.");
                        _this.$logger.info("Applying changes...");
                        var platformSpecificLiveSyncService = _this.resolvePlatformSpecificLiveSyncService(platform, device, platformSpecificLiveSyncServices);
                        platformSpecificLiveSyncService.restartApplication(deviceAppData, localToDevicePaths).wait();
                        _this.$logger.info("Successfully synced application " + deviceAppData.appIdentifier + ".");
                    }
                }).future()();
            };
            _this.$devicesServices.execute(action).wait();
        }).future()();
    };
    UsbLiveSyncServiceBase.prototype.batchLiveSync = function (platform, filePath, appIdentifier, projectFilesPath, platformSpecificLiveSyncServices, notInstalledAppOnDeviceAction, beforeLiveSyncAction, beforeBatchLiveSyncAction) {
        var _this = this;
        if (!this.batch || !this.batch.syncPending) {
            this.batch = new SyncBatch(this.$logger, this.$dispatcher, function (filesToSync) {
                _this.preparePlatformForSync(platform);
                _this.syncCore(platform, filesToSync, appIdentifier, projectFilesPath, platformSpecificLiveSyncServices, notInstalledAppOnDeviceAction, beforeLiveSyncAction).wait();
            });
        }
        this.$dispatcher.dispatch(function () { return (function () {
            var fileToSync = beforeBatchLiveSyncAction ? beforeBatchLiveSyncAction(filePath).wait() : filePath;
            if (fileToSync) {
                _this.batch.addFile(fileToSync);
            }
        }).future()(); });
    };
    UsbLiveSyncServiceBase.prototype.batchSimulatorLiveSync = function (appIdentifier, projectFilesPath, filePath, notRunningiOSSimulatorAction, iOSSimulatorRelativeToProjectBasePathAction) {
        var _this = this;
        if (!this.batch || !this.batch.syncPending) {
            this.batch = new SyncBatch(this.$logger, this.$dispatcher, function (filesToSync) {
                _this.$iOSEmulatorServices.syncFiles(appIdentifier, projectFilesPath, filesToSync, notRunningiOSSimulatorAction, iOSSimulatorRelativeToProjectBasePathAction);
            });
        }
        this.batch.addFile(filePath);
    };
    UsbLiveSyncServiceBase.prototype.preparePlatformForSync = function (platform) {
    };
    UsbLiveSyncServiceBase.prototype.isFileExcluded = function (path, exclusionList, projectDir) {
        return !!_.find(exclusionList, function (pattern) { return minimatch(path, pattern, { nocase: true }); });
    };
    UsbLiveSyncServiceBase.prototype.getProjectFileInfo = function (fileName) {
        var parsed = this.parseFile(fileName, this.$mobileHelper.platformNames, this.$devicesServices.platform);
        if (!parsed) {
            parsed = this.parseFile(fileName, ["debug", "release"], "debug");
        }
        return parsed || {
            fileName: fileName,
            onDeviceName: fileName,
            shouldIncludeFile: true
        };
    };
    UsbLiveSyncServiceBase.prototype.parseFile = function (fileName, validValues, value) {
        var regex = util.format("^(.+?)[.](%s)([.].+?)$", validValues.join("|"));
        var parsed = fileName.match(new RegExp(regex, "i"));
        if (parsed) {
            return {
                fileName: fileName,
                onDeviceName: parsed[1] + parsed[3],
                shouldIncludeFile: parsed[2].toLowerCase() === value.toLowerCase(),
                value: value
            };
        }
        return undefined;
    };
    UsbLiveSyncServiceBase.prototype.resolvePlatformSpecificLiveSyncService = function (platform, device, platformSpecificLiveSyncServices) {
        return this.$injector.resolve(platformSpecificLiveSyncServices[platform.toLowerCase()], { _device: device });
    };
    return UsbLiveSyncServiceBase;
})();
exports.UsbLiveSyncServiceBase = UsbLiveSyncServiceBase;
