///<reference path="../.d.ts"/>
"use strict";
var os_1 = require("os");
var helpers = require("../common/helpers");
var semver = require("semver");
var DoctorService = (function () {
    function DoctorService($androidToolsInfo, $hostInfo, $logger, $sysInfo) {
        this.$androidToolsInfo = $androidToolsInfo;
        this.$hostInfo = $hostInfo;
        this.$logger = $logger;
        this.$sysInfo = $sysInfo;
    }
    DoctorService.prototype.printWarnings = function () {
        var result = false;
        var sysInfo = this.$sysInfo.getSysInfo();
        if (!sysInfo.adbVer) {
            this.$logger.warn("WARNING: adb from the Android SDK is not installed or is not configured properly.");
            this.$logger.out("For Android-related operations, the NativeScript CLI will use a built-in version of adb." + os_1.EOL
                + "To avoid possible issues with the native Android emulator, Genymotion or connected" + os_1.EOL
                + "Android devices, verify that you have installed the latest Android SDK and" + os_1.EOL
                + "its dependencies as described in http://developer.android.com/sdk/index.html#Requirements" + os_1.EOL);
            this.printPackageManagerTip();
            result = true;
        }
        if (!sysInfo.androidInstalled) {
            this.$logger.warn("WARNING: The Android SDK is not installed or is not configured properly.");
            this.$logger.out("You will not be able to build your projects for Android and run them in the native emulator." + os_1.EOL
                + "To be able to build for Android and run apps in the native emulator, verify that you have" + os_1.EOL
                + "installed the latest Android SDK and its dependencies as described in http://developer.android.com/sdk/index.html#Requirements" + os_1.EOL);
            this.printPackageManagerTip();
            result = true;
        }
        if (this.$hostInfo.isDarwin && !sysInfo.xcodeVer) {
            this.$logger.warn("WARNING: Xcode is not installed or is not configured properly.");
            this.$logger.out("You will not be able to build your projects for iOS or run them in the iOS Simulator." + os_1.EOL
                + "To be able to build for iOS and run apps in the native emulator, verify that you have installed Xcode." + os_1.EOL);
            result = true;
        }
        if (!this.$hostInfo.isDarwin) {
            this.$logger.warn("WARNING: You can work with iOS only on Mac OS X systems.");
            this.$logger.out("To be able to work with iOS devices and projects, you need Mac OS X Mavericks or later." + os_1.EOL);
            result = true;
        }
        if (!sysInfo.javaVer) {
            this.$logger.warn("WARNING: The Java Development Kit (JDK) is not installed or is not configured properly.");
            this.$logger.out("You will not be able to work with the Android SDK and you might not be able" + os_1.EOL
                + "to perform some Android-related operations. To ensure that you can develop and" + os_1.EOL
                + "test your apps for Android, verify that you have installed the JDK as" + os_1.EOL
                + "described in http://docs.oracle.com/javase/8/docs/technotes/guides/install/install_overview.html (for JDK 8)" + os_1.EOL
                + "or http://docs.oracle.com/javase/7/docs/webnotes/install/ (for JDK 7)." + os_1.EOL);
            result = true;
        }
        if (!sysInfo.gradleVer) {
            this.$logger.warn("WARNING: Gradle is not installed or is not configured properly.");
            this.$logger.out("You will not be able to build your projects for Android or run them in the emulator or on a connected device." + os_1.EOL
                + "To be able to build for Android and run apps in the emulator or on a connected device, verify that you have installed Gradle.");
            result = true;
        }
        if (sysInfo.gradleVer && !this.isGradleVersionSupported(sysInfo.gradleVer)) {
            this.$logger.warn("WARNING: Gradle version is lower than " + DoctorService.MIN_SUPPORTED_GRADLE_VERSION + ".");
            this.$logger.out("You will not be able to build your projects for Android or run them in the emulator or on a connected device." + os_1.EOL
                + ("To be able to build for Android and run apps in the emulator or on a connected device, verify that you have at least " + DoctorService.MIN_SUPPORTED_GRADLE_VERSION + " version installed."));
            result = true;
        }
        if (!sysInfo.javacVersion) {
            this.$logger.warn("WARNING: Javac is not installed or is not configured properly.");
            this.$logger.out("You will not be able to build your projects for Android." + os_1.EOL
                + "To be able to build for Android, verify that you have installed The Java Development Kit (JDK) and configured it according to system requirements as" + os_1.EOL +
                " described in https://github.com/NativeScript/nativescript-cli#system-requirements.");
            result = true;
        }
        if (!sysInfo.cocoapodVer) {
            this.$logger.warn("WARNING: CocoaPod is not installed or is not configured properly.");
            this.$logger.out("You will not be able to build your projects for iOS if they contain plugin with CocoaPod file." + os_1.EOL
                + "To be able to build such projects, verify that you have installed CocoaPod.");
            result = true;
        }
        if (sysInfo.cocoapodVer && semver.lt(sysInfo.cocoapodVer, DoctorService.MIN_SUPPORTED_POD_VERSION)) {
            this.$logger.warn("WARNING: CocoaPod version is lower than " + DoctorService.MIN_SUPPORTED_POD_VERSION);
            this.$logger.out("You will not be able to build your projects for iOS if they contain plugin with CocoaPod file." + os_1.EOL
                + ("To be able to build such projects, verify that you have at least " + DoctorService.MIN_SUPPORTED_POD_VERSION + " version installed."));
            result = true;
        }
        var androidToolsIssues = this.$androidToolsInfo.validateInfo().wait();
        return result || androidToolsIssues;
    };
    DoctorService.prototype.printPackageManagerTip = function () {
        if (this.$hostInfo.isWindows) {
            this.$logger.out("TIP: To avoid setting up the necessary environment variables, you can use the chocolatey package manager to install the Android SDK and its dependencies." + os_1.EOL);
        }
        else if (this.$hostInfo.isDarwin) {
            this.$logger.out("TIP: To avoid setting up the necessary environment variables, you can use the Homebrew package manager to install the Android SDK and its dependencies." + os_1.EOL);
        }
    };
    DoctorService.prototype.isGradleVersionSupported = function (gradleVersion) {
        var minSupportedVersion = DoctorService.MIN_SUPPORTED_GRADLE_VERSION;
        var requiredLength = _.max([gradleVersion.split(".").length, minSupportedVersion.split(".").length]);
        gradleVersion = helpers.appendZeroesToVersion(gradleVersion, requiredLength);
        minSupportedVersion = helpers.appendZeroesToVersion(minSupportedVersion, requiredLength);
        return helpers.versionCompare(gradleVersion, minSupportedVersion) !== -1;
    };
    DoctorService.MIN_SUPPORTED_GRADLE_VERSION = "2.3";
    DoctorService.MIN_SUPPORTED_POD_VERSION = "0.38.2";
    return DoctorService;
})();
$injector.register("doctorService", DoctorService);
